@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2016
+# Copyright (C) 2015-2017
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@ Telegram bots intuitive."""
 import logging
 import os
 import ssl
+import warnings
 from threading import Thread, Lock, current_thread, Event
 from time import sleep
 import subprocess
@@ -61,6 +62,11 @@ class Updater(object):
         bot (Optional[Bot]): A pre-initialized bot instance. If a pre-initizlied bot is used, it is
             the user's responsibility to create it using a `Request` instance with a large enough
             connection pool.
+        user_sig_handler (Optional[function]): Takes ``signum, frame`` as positional arguments.
+            This will be called when a signal is received, defaults are (SIGINT, SIGTERM, SIGABRT)
+            setable with Updater.idle(stop_signals=(signals))
+        request_kwargs (Optional[dict]): Keyword args to control the creation of a request object
+            (ignored if `bot` argument is used).
 
     Raises:
         ValueError: If both `token` and `bot` are passed or none of them.
@@ -68,7 +74,14 @@ class Updater(object):
     """
     _request = None
 
-    def __init__(self, token=None, base_url=None, workers=4, bot=None):
+    def __init__(self,
+                 token=None,
+                 base_url=None,
+                 workers=4,
+                 bot=None,
+                 user_sig_handler=None,
+                 request_kwargs=None):
+
         if (token is None) and (bot is None):
             raise ValueError('`token` or `bot` must be passed')
         if (token is not None) and (bot is not None):
@@ -83,8 +96,13 @@ class Updater(object):
             # * 1 for polling Updater (even if webhook is used, we can spare a connection)
             # * 1 for JobQueue
             # * 1 for main thread
-            self._request = Request(con_pool_size=workers + 4)
+            if request_kwargs is None:
+                request_kwargs = {}
+            if 'con_pool_size' not in request_kwargs:
+                request_kwargs['con_pool_size'] = workers + 4
+            self._request = Request(**request_kwargs)
             self.bot = Bot(token, base_url, request=self._request)
+        self.user_sig_handler = user_sig_handler
         self.update_queue = Queue()
         self.job_queue = JobQueue(self.bot)
         self.__exception_event = Event()
@@ -122,9 +140,11 @@ class Updater(object):
     def start_polling(self,
                       poll_interval=0.0,
                       timeout=10,
-                      network_delay=5.,
+                      network_delay=None,
                       clean=False,
-                      bootstrap_retries=0):
+                      bootstrap_retries=0,
+                      read_latency=2.,
+                      allowed_updates=None):
         """
         Starts polling updates from Telegram.
 
@@ -134,7 +154,8 @@ class Updater(object):
 
             timeout (Optional[float]): Passed to Bot.getUpdates
 
-            network_delay (Optional[float]): Passed to Bot.getUpdates
+            network_delay: Deprecated. Will be honoured as `read_latency` for a while but will be
+                removed in the future.
 
             clean (Optional[bool]): Whether to clean any pending updates on Telegram servers before
               actually starting to poll. Default is False.
@@ -143,13 +164,24 @@ class Updater(object):
               will retry on failures on the Telegram server.
 
                 |   < 0 - retry indefinitely
-                |   0 - no retries (default)
+                |     0 - no retries (default)
                 |   > 0 - retry up to X times
 
+            allowed_updates (Optional[list[str]]): Passed to Bot.getUpdates
+
+            read_latency (Optional[float|int]): Grace time in seconds for receiving the reply from
+                server. Will be added to the `timeout` value and used as the read timeout from
+                server (Default: 2).
+
+
         Returns:
             Queue: The update queue that can be filled from the main thread
 
         """
+        if network_delay is not None:
+            warnings.warn('network_delay is deprecated, use read_latency instead')
+            read_latency = network_delay
+
         with self.__lock:
             if not self.running:
                 self.running = True
@@ -158,7 +190,7 @@ class Updater(object):
                 self.job_queue.start()
                 self._init_thread(self.dispatcher.start, "dispatcher")
                 self._init_thread(self._start_polling, "updater", poll_interval, timeout,
-                                  network_delay, bootstrap_retries, clean)
+                                  read_latency, bootstrap_retries, clean, allowed_updates)
 
                 # Return the update queue so the main thread can insert updates
                 return self.update_queue
@@ -171,7 +203,8 @@ class Updater(object):
                       key=None,
                       clean=False,
                       bootstrap_retries=0,
-                      webhook_url=None):
+                      webhook_url=None,
+                      allowed_updates=None):
         """
         Starts a small http server to listen for updates via webhook. If cert
         and key are not provided, the webhook will be started directly on
@@ -194,9 +227,10 @@ class Updater(object):
                 |   < 0 - retry indefinitely
                 |   0 - no retries (default)
                 |   > 0 - retry up to X times
-            webhook_url (Optional[str]): Explicitly specifiy the webhook url.
+            webhook_url (Optional[str]): Explicitly specify the webhook url.
                 Useful behind NAT, reverse proxy, etc. Default is derived from
                 `listen`, `port` & `url_path`.
+            allowed_updates (Optional[list[str]]): Passed to Bot.setWebhook
 
         Returns:
             Queue: The update queue that can be filled from the main thread
@@ -210,12 +244,13 @@ class Updater(object):
                 self.job_queue.start()
                 self._init_thread(self.dispatcher.start, "dispatcher"),
                 self._init_thread(self._start_webhook, "updater", listen, port, url_path, cert,
-                                  key, bootstrap_retries, clean, webhook_url)
+                                  key, bootstrap_retries, clean, webhook_url, allowed_updates)
 
                 # Return the update queue so the main thread can insert updates
                 return self.update_queue
 
-    def _start_polling(self, poll_interval, timeout, network_delay, bootstrap_retries, clean):
+    def _start_polling(self, poll_interval, timeout, read_latency, bootstrap_retries, clean,
+                       allowed_updates):
         """
         Thread target of thread 'updater'. Runs in background, pulls
         updates from Telegram and inserts them in the update queue of the
@@ -225,12 +260,15 @@ class Updater(object):
         cur_interval = poll_interval
         self.logger.debug('Updater thread started')
 
-        self._bootstrap(bootstrap_retries, clean=clean, webhook_url='')
+        self._bootstrap(bootstrap_retries, clean=clean, webhook_url='', allowed_updates=None)
 
         while self.running:
             try:
                 updates = self.bot.getUpdates(
-                    self.last_update_id, timeout=timeout, network_delay=network_delay)
+                    self.last_update_id,
+                    timeout=timeout,
+                    read_latency=read_latency,
+                    allowed_updates=allowed_updates)
             except RetryAfter as e:
                 self.logger.info(str(e))
                 cur_interval = 0.5 + e.retry_after
@@ -270,7 +308,7 @@ class Updater(object):
         return current_interval
 
     def _start_webhook(self, listen, port, url_path, cert, key, bootstrap_retries, clean,
-                       webhook_url):
+                       webhook_url, allowed_updates):
         self.logger.debug('Updater thread started')
         use_ssl = cert is not None and key is not None
         if not url_path.startswith('/'):
@@ -291,7 +329,8 @@ class Updater(object):
                 max_retries=bootstrap_retries,
                 clean=clean,
                 webhook_url=webhook_url,
-                cert=open(cert, 'rb'))
+                cert=open(cert, 'rb'),
+                allowed_updates=allowed_updates)
         elif clean:
             self.logger.warning("cleaning updates is not supported if "
                                 "SSL-termination happens elsewhere; skipping")
@@ -321,18 +360,19 @@ class Updater(object):
     def _gen_webhook_url(listen, port, url_path):
         return 'https://{listen}:{port}{path}'.format(listen=listen, port=port, path=url_path)
 
-    def _bootstrap(self, max_retries, clean, webhook_url, cert=None):
+    def _bootstrap(self, max_retries, clean, webhook_url, allowed_updates, cert=None):
         retries = 0
         while 1:
 
             try:
                 if clean:
                     # Disable webhook for cleaning
-                    self.bot.setWebhook(webhook_url='')
+                    self.bot.deleteWebhook()
                     self._clean_updates()
                     sleep(1)
 
-                self.bot.setWebhook(webhook_url=webhook_url, certificate=cert)
+                self.bot.setWebhook(
+                    url=webhook_url, certificate=cert, allowed_updates=allowed_updates)
             except (Unauthorized, InvalidToken):
                 raise
             except TelegramError:
@@ -397,6 +437,8 @@ class Updater(object):
         self.is_idle = False
         if self.running:
             self.stop()
+            if self.user_sig_handler:
+                self.user_sig_handler(signum, frame)
         else:
             self.logger.warning('Exiting immediately!')
             import os
